@@ -19,7 +19,7 @@
 # ============================================================================
 # Name        : FeedingIt.py
 # Author      : Yves Marcoz
-# Version     : 0.2.2
+# Version     : 0.4.1
 # Description : Simple RSS Reader
 # ============================================================================
 
@@ -40,13 +40,20 @@ from portrait import FremantleRotation
 import threading
 import thread
 from feedingitdbus import ServerObject
+from config import Config
 
 from rss import *
 from opml import GetOpmlData, ExportOpmlData
    
+import socket
+timeout = 5
+socket.setdefaulttimeout(timeout)
+
+CONFIGDIR="/home/user/.feedingit/"
+
 class AddWidgetWizard(hildon.WizardDialog):
     
-    def __init__(self, parent, urlIn):
+    def __init__(self, parent, urlIn, titleIn=None):
         # Create a Notebook
         self.notebook = gtk.Notebook()
 
@@ -56,6 +63,8 @@ class AddWidgetWizard(hildon.WizardDialog):
         label = gtk.Label("Enter Feed Name:")
         vbox.pack_start(label)
         vbox.pack_start(self.nameEntry)
+        if not titleIn == None:
+            self.nameEntry.set_text(titleIn)
         self.notebook.append_page(vbox, None)
         
         self.urlEntry = hildon.Entry(gtk.HILDON_SIZE_AUTO)
@@ -102,50 +111,81 @@ class AddWidgetWizard(hildon.WizardDialog):
             return True
 
 class GetImage(threading.Thread):
-    def __init__(self, url):
+    def __init__(self, url, stream):
         threading.Thread.__init__(self)
         self.url = url
+        self.stream = stream
     
     def run(self):
         f = urllib2.urlopen(self.url)
-        self.data = f.read()
+        data = f.read()
         f.close()
+        self.stream.write(data)
+        self.stream.close()
 
+class ImageDownloader():
+    def __init__(self):
+        self.images = []
+        self.downloading = False
+        
+    def queueImage(self, url, stream):
+        self.images.append((url, stream))
+        if not self.downloading:
+            self.downloading = True
+            gobject.timeout_add(50, self.checkQueue)
+        
+    def checkQueue(self):
+        for i in range(4-threading.activeCount()):
+            if len(self.images) > 0:
+                (url, stream) = self.images.pop() 
+                GetImage(url, stream).start()
+        if len(self.images)>0:
+            gobject.timeout_add(200, self.checkQueue)
+        else:
+            self.downloading=False
+            
+    def stopAll(self):
+        self.images = []
+            
         
 class Download(threading.Thread):
-    def __init__(self, listing, key):
+    def __init__(self, listing, key, config):
         threading.Thread.__init__(self)
         self.listing = listing
         self.key = key
+        self.config = config
         
     def run (self):
-        self.listing.updateFeed(self.key)
+        self.listing.updateFeed(self.key, self.config.getExpiry())
 
         
-class DownloadDialog():
-    def __init__(self, parent, listing, listOfKeys):
+class DownloadBar(gtk.ProgressBar):
+    def __init__(self, parent, listing, listOfKeys, config):
+        gtk.ProgressBar.__init__(self)
         self.listOfKeys = listOfKeys[:]
         self.listing = listing
         self.total = len(self.listOfKeys)
-        self.current = 0            
+        self.config = config
+        self.current = 0
         
         if self.total>0:
-            self.progress = gtk.ProgressBar()
-            self.waitingWindow = hildon.Note("cancel", parent, "Downloading",
-                                 progressbar=self.progress)
-            self.progress.set_text("Downloading")
+            #self.progress = gtk.ProgressBar()
+            #self.waitingWindow = hildon.Note("cancel", parent, "Downloading",
+            #                     progressbar=self.progress)
+            self.set_text("Downloading")
             self.fraction = 0
-            self.progress.set_fraction(self.fraction)
+            self.set_fraction(self.fraction)
+            self.show_all()
             # Create a timeout
             self.timeout_handler_id = gobject.timeout_add(50, self.update_progress_bar)
-            self.waitingWindow.show_all()
-            response = self.waitingWindow.run()
-            self.listOfKeys = []
-            while threading.activeCount() > 1:
+            #self.waitingWindow.show_all()
+            #response = self.waitingWindow.run()
+            #self.listOfKeys = []
+            #while threading.activeCount() > 1:
                 # Wait for current downloads to finish
-                time.sleep(0.5)
-            self.waitingWindow.destroy()
-        
+            #    time.sleep(0.1)
+            #self.waitingWindow.destroy()
+
     def update_progress_bar(self):
         #self.progress_bar.pulse()
         if threading.activeCount() < 4:
@@ -154,18 +194,22 @@ class DownloadDialog():
             fin = self.total - k - x
             fraction = float(fin)/float(self.total) + float(x)/(self.total*2.)
             #print x, k, fin, fraction
-            self.progress.set_fraction(fraction)
+            self.set_fraction(fraction)
 
             if len(self.listOfKeys)>0:
                 self.current = self.current+1
                 key = self.listOfKeys.pop()
-                download = Download(self.listing, key)
-                download.start()
+                if not self.listing.getCurrentlyDisplayedFeed() == key:
+                    # Check if the feed is being displayed
+                    download = Download(self.listing, key, self.config)
+                    download.start()
                 return True
             elif threading.activeCount() > 1:
                 return True
             else:
-                self.waitingWindow.destroy()
+                #self.waitingWindow.destroy()
+                #self.destroy()
+                self.emit("download-done", "success")
                 return False 
         return True
     
@@ -186,6 +230,11 @@ class SortList(gtk.Dialog):
         button.set_label("Move Down")
         button.connect("clicked", self.buttonDown)
         self.vbox2.pack_start(button, expand=False, fill=False)
+
+        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        button.set_label("Edit Feed")
+        button.connect("clicked", self.buttonEdit)
+        self.vbox2.pack_start(button, expand=False, fill=False)
         
         button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
         button.set_label("Delete")
@@ -274,15 +323,31 @@ class SortList(gtk.Dialog):
             self.listing.removeFeed(key)
         self.refreshList()
 
+    def buttonEdit(self, button):
+        key = self.getSelectedItem()
+        if not key == None:
+            wizard = AddWidgetWizard(self, self.listing.getFeedUrl(key), self.listing.getFeedTitle(key))
+            ret = wizard.run()
+            if ret == 2:
+                (title, url) = wizard.getData()
+                if (not title == '') and (not url == ''):
+                    self.listing.editFeed(key, title, url)
+            wizard.destroy()
+        self.refreshList()
+
     def buttonDone(self, *args):
         self.destroy()
                
 
 class DisplayArticle(hildon.StackableWindow):
-    def __init__(self, title, text, index):
+    def __init__(self, title, text, link, index, key, listing):
         hildon.StackableWindow.__init__(self)
+        self.imageDownloader = ImageDownloader()
+        self.listing=listing
+        self.key = key
         self.index = index
         self.text = text
+        self.link = link
         self.set_title(title)
         self.images = []
         
@@ -294,8 +359,11 @@ class DisplayArticle(hildon.StackableWindow):
         self.gestureId = self.pannable_article.connect('horizontal-movement', self.gesture)
         self.document = gtkhtml2.Document()
         self.view.set_document(self.document)
+        
         self.document.connect("link_clicked", self._signal_link_clicked)
-        self.document.connect("request-url", self._signal_request_url)
+        if not key == "1295627ef630df9d239abeb0ba631c3f":
+            # Do not download images if the feed is "Archived Articles"
+            self.document.connect("request-url", self._signal_request_url)
         self.document.clear()
         self.document.open_stream("text/html")
         self.document.write_stream(self.text)
@@ -306,44 +374,60 @@ class DisplayArticle(hildon.StackableWindow):
         button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
         button.set_label("Allow Horizontal Scrolling")
         button.connect("clicked", self.horiz_scrolling_button)
-        
         menu.append(button)
+        
+        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        button.set_label("Open in Browser")
+        button.connect("clicked", self._signal_link_clicked, self.link)
+        menu.append(button)
+        
+        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        button.set_label("Add to Archived Articles")
+        button.connect("clicked", self.archive_button)
+        menu.append(button)
+        
         self.set_app_menu(menu)
         menu.show_all()
         
         self.add(self.pannable_article)
         
-        self.show_all()
+        self.pannable_article.show_all()
 
         self.destroyId = self.connect("destroy", self.destroyWindow)
-        self.timeout_handler_id = gobject.timeout_add(300, self.reloadArticle)
+        #self.timeout_handler_id = gobject.timeout_add(300, self.reloadArticle)
 
     def gesture(self, widget, direction, startx, starty):
         if (direction == 3):
             self.emit("article-next", self.index)
         if (direction == 2):
             self.emit("article-previous", self.index)
-        self.timeout_handler_id = gobject.timeout_add(200, self.destroyWindow)
+        #self.timeout_handler_id = gobject.timeout_add(200, self.destroyWindow)
 
     def destroyWindow(self, *args):
+        self.disconnect(self.destroyId)
         self.emit("article-closed", self.index)
+        self.imageDownloader.stopAll()
         self.destroy()
         
     def horiz_scrolling_button(self, *widget):
         self.pannable_article.disconnect(self.gestureId)
         self.pannable_article.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
         
-    def reloadArticle(self, *widget):
-        if threading.activeCount() > 1:
+    def archive_button(self, *widget):
+        # Call the listing.addArchivedArticle
+        self.listing.addArchivedArticle(self.key, self.index)
+        
+    #def reloadArticle(self, *widget):
+    #    if threading.activeCount() > 1:
             # Image thread are still running, come back in a bit
-            return True
-        else:
-            for (stream, imageThread) in self.images:
-                imageThread.join()
-                stream.write(imageThread.data)
-                stream.close()
-            return False
-        self.show_all()
+    #        return True
+    #    else:
+    #        for (stream, imageThread) in self.images:
+    #            imageThread.join()
+    #            stream.write(imageThread.data)
+    #            stream.close()
+    #        return False
+    #    self.show_all()
 
     def _signal_link_clicked(self, object, link):
         bus = dbus.SystemBus()
@@ -352,25 +436,33 @@ class DisplayArticle(hildon.StackableWindow):
         iface.open_new_window(link)
 
     def _signal_request_url(self, object, url, stream):
-        imageThread = GetImage(url)
-        imageThread.start()
-        self.images.append((stream, imageThread))
+        #print url
+        self.imageDownloader.queueImage(url, stream)
+        #imageThread = GetImage(url)
+        #imageThread.start()
+        #self.images.append((stream, imageThread))
 
 
 class DisplayFeed(hildon.StackableWindow):
-    def __init__(self, listing, feed, title, key):
+    def __init__(self, listing, feed, title, key, config):
         hildon.StackableWindow.__init__(self)
         self.listing = listing
         self.feed = feed
         self.feedTitle = title
         self.set_title(title)
         self.key=key
+        self.config = config
+        
+        self.listing.setCurrentlyDisplayedFeed(self.key)
+        
+        self.disp = False
         
         menu = hildon.AppMenu()
-        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-        button.set_label("Update Feed")
-        button.connect("clicked", self.button_update_clicked)
-        menu.append(button)
+        #button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        #button.set_label("Update Feed")
+        #button.connect("clicked", self.button_update_clicked)
+        #menu.append(button)
+        
         button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
         button.set_label("Mark All As Read")
         button.connect("clicked", self.buttonReadAllClicked)
@@ -385,7 +477,8 @@ class DisplayFeed(hildon.StackableWindow):
     def destroyWindow(self, *args):
         self.emit("feed-closed", self.key)
         self.destroy()
-        self.feed.saveFeed()
+        gobject.idle_add(self.feed.saveFeed, CONFIGDIR)
+        self.listing.closeCurrentlyDisplayedFeed()
 
     def displayFeed(self):
         self.vboxFeed = gtk.VBox(False, 10)
@@ -399,10 +492,10 @@ class DisplayFeed(hildon.StackableWindow):
             label = button.child
             if self.feed.isEntryRead(index):
                 #label.modify_font(pango.FontDescription("sans 16"))
-                label.modify_font(pango.FontDescription(self.listing.getReadFont()))
+                label.modify_font(pango.FontDescription(self.config.getReadFont()))
             else:
                 #print self.listing.getFont() + " bold"
-                label.modify_font(pango.FontDescription(self.listing.getUnreadFont()))
+                label.modify_font(pango.FontDescription(self.config.getUnreadFont()))
                 #label.modify_font(pango.FontDescription("sans bold 23"))
                 #"sans bold 16"
             label.set_line_wrap(True)
@@ -419,59 +512,105 @@ class DisplayFeed(hildon.StackableWindow):
         
     def clear(self):
         self.remove(self.pannableFeed)
+
+    def button_clicked(self, button, index, previous=False, next=False):
+        newDisp = DisplayArticle(self.feedTitle, self.feed.getArticle(index), self.feed.getLink(index), index, self.key, self.listing)
+        stack = hildon.WindowStack.get_default()
+        if previous:
+            tmp = stack.peek()
+            stack.pop_and_push(1, newDisp, tmp)
+            newDisp.show()
+            gobject.timeout_add(200, self.destroyArticle, tmp)
+            #print "previous"
+            self.disp = newDisp
+            
+            #stack.push(tmp)
+            #if not self.disp == False:
+            #   self.disp.destroyWindow()
+        elif next:
+            #print type(self.disp).__name__
+
+                #self.disp.destroyWindow()
+                #stack.pop_and_push(1,newDisp)
+            #else:
+            #    stack.push(newDisp)
+            #self.disp = newDisp
+            newDisp.show_all()
+            if type(self.disp).__name__ == "DisplayArticle":
+                gobject.timeout_add(200, self.destroyArticle, self.disp)
+            self.disp = newDisp
+            #self.disp.show_all()
+            #if not self.disp == False:
+            #    self.disp.destroyWindow()
+        else:
+            self.disp = newDisp
+            self.disp.show_all()
         
-    def button_clicked(self, button, index):
-        self.disp = DisplayArticle(self.feedTitle, self.feed.getArticle(index), index)
         self.ids = []
         self.ids.append(self.disp.connect("article-closed", self.onArticleClosed))
         self.ids.append(self.disp.connect("article-next", self.nextArticle))
         self.ids.append(self.disp.connect("article-previous", self.previousArticle))
 
+    def destroyArticle(self, handle):
+        handle.destroyWindow()
+
     def nextArticle(self, object, index):
         label = self.buttons[index].child
-        label.modify_font(pango.FontDescription(self.listing.getReadFont()))
+        label.modify_font(pango.FontDescription(self.config.getReadFont()))
         index = (index+1) % self.feed.getNumberOfEntries()
-        self.button_clicked(object, index)
+        self.button_clicked(object, index, next=True)
 
     def previousArticle(self, object, index):
         label = self.buttons[index].child
-        label.modify_font(pango.FontDescription(self.listing.getReadFont()))
+        label.modify_font(pango.FontDescription(self.config.getReadFont()))
         index = (index-1) % self.feed.getNumberOfEntries()
-        self.button_clicked(object, index)
+        self.button_clicked(object, index, previous=True)
 
     def onArticleClosed(self, object, index):
         label = self.buttons[index].child
-        label.modify_font(pango.FontDescription(self.listing.getReadFont()))
+        label.modify_font(pango.FontDescription(self.config.getReadFont()))
         self.buttons[index].show()
 
-    def button_update_clicked(self, button):
-        disp = DownloadDialog(self, self.listing, [self.key,] )       
+    #def button_update_clicked(self, button):
+    #    bar = DownloadBar(self, self.listing, [self.key,], self.config )       
         #self.feed.updateFeed()
-        self.clear()
-        self.displayFeed()
+    #    self.clear()
+    #    self.displayFeed()
         
     def buttonReadAllClicked(self, button):
         for index in range(self.feed.getNumberOfEntries()):
             self.feed.setEntryRead(index)
             label = self.buttons[index].child
-            label.modify_font(pango.FontDescription(self.listing.getReadFont()))
+            label.modify_font(pango.FontDescription(self.config.getReadFont()))
             self.buttons[index].show()
 
 
 class FeedingIt:
     def __init__(self):
-        self.listing = Listing()
-        
         # Init the windows
         self.window = hildon.StackableWindow()
         self.window.set_title("FeedingIt")
-        FremantleRotation("FeedingIt", main_window=self.window)
+        hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
+        self.pannableListing = gtk.Label("Loading...")
+        self.window.add(self.pannableListing)
+        self.window.show_all()
+        self.config = Config(self.window, CONFIGDIR+"config.ini")
+        gobject.idle_add(self.createWindow)
+        
+    def createWindow(self):
+        self.listing = Listing(CONFIGDIR)
+        
+        self.downloadDialog = False
+        self.orientation = FremantleRotation("FeedingIt", main_window=self.window)
+        self.orientation.set_mode(self.config.getOrientation())
+        
         menu = hildon.AppMenu()
         # Create a button and add it to the menu
         button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
         button.set_label("Update All Feeds")
         button.connect("clicked", self.button_update_clicked, "All")
         menu.append(button)
+        
         button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
         button.set_label("Add Feed")
         button.connect("clicked", self.button_add_clicked)
@@ -483,8 +622,8 @@ class FeedingIt:
         menu.append(button)
 
         button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-        button.set_label("Listing Font Size")
-        button.connect("clicked", self.button_font_clicked)
+        button.set_label("Preferences")
+        button.connect("clicked", self.button_preferences_clicked)
         menu.append(button)
        
         button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
@@ -500,10 +639,14 @@ class FeedingIt:
         self.window.set_app_menu(menu)
         menu.show_all()
         
-        self.feedWindow = hildon.StackableWindow()
-        self.articleWindow = hildon.StackableWindow()
+        #self.feedWindow = hildon.StackableWindow()
+        #self.articleWindow = hildon.StackableWindow()
+
+        self.displayListing()
+        self.autoupdate = False
+        self.checkAutoUpdate()
+        hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
 
-        self.displayListing() 
 
     def button_export_clicked(self, button):
         opml = ExportOpmlData(self.window, self.listing)
@@ -529,41 +672,26 @@ class FeedingIt:
             (title, url) = wizard.getData()
             if (not title == '') and (not url == ''): 
                self.listing.addFeed(title, url)
+                   
         wizard.destroy()
         self.displayListing()
         
     def button_update_clicked(self, button, key):
-        disp = DownloadDialog(self.window, self.listing, self.listing.getListOfFeeds() )           
+        if not type(self.downloadDialog).__name__=="DownloadBar":
+            self.downloadDialog = DownloadBar(self.window, self.listing, self.listing.getListOfFeeds(), self.config )
+            self.downloadDialog.connect("download-done", self.onDownloadsDone)
+            self.vboxListing.pack_start(self.downloadDialog)
+            self.pannableListing.show_all()
+        #self.displayListing()
+
+    def onDownloadsDone(self, *widget):
+        self.downloadDialog.destroy()
+        self.downloadDialog = False
         self.displayListing()
 
-    def button_font_clicked(self, button):
-        self.pickerDialog = hildon.PickerDialog(self.window)
-        #HildonPickerDialog
-        selector = self.create_selector()
-        self.pickerDialog.set_selector(selector)
-        self.pickerDialog.show_all()
-        
-    def create_selector(self):
-        selector = hildon.TouchSelector(text=True)
-        # Selection multiple
-        #selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)
-        self.mapping = {}
-
-        current_size = self.listing.getFontSize()
-        index = 0
-        for size in range(12,24):
-            iter = selector.append_text(str(size))
-            if str(size) == current_size: 
-                selector.set_active(0, index)
-            index += 1
-        selector.connect("changed", self.selection_changed)
-        return selector
-
-    def selection_changed(self, widget, data):
-        current_selection = widget.get_current_text()
-        if current_selection:
-            self.listing.setFont(current_selection)
-        self.pickerDialog.destroy()
+    def button_preferences_clicked(self, button):
+        dialog = self.config.createDialog()
+        dialog.connect("destroy", self.prefsClosed)
 
     def show_confirmation_note(self, parent, title):
         note = hildon.Note("confirmation", parent, "Are you sure you want to delete " + title +"?")
@@ -586,7 +714,9 @@ class FeedingIt:
         self.pannableListing.add_with_viewport(self.vboxListing)
 
         self.buttons = {}
-        for key in self.listing.getListOfFeeds():
+        list = self.listing.getListOfFeeds()[:]
+        list.reverse()
+        for key in list:
             #button = gtk.Button(item)
             button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT,
                               hildon.BUTTON_ARRANGEMENT_VERTICAL)
@@ -594,31 +724,56 @@ class FeedingIt:
                             + str(self.listing.getFeedNumberOfUnreadItems(key)) + " Unread Items")
             button.set_alignment(0,0,1,1)
             button.connect("clicked", self.buttonFeedClicked, self, self.window, key)
-            self.vboxListing.pack_start(button, expand=False)
+            self.vboxListing.pack_end(button) #, expand=False)
             self.buttons[key] = button
+            
+        if type(self.downloadDialog).__name__=="DownloadBar":
+            self.vboxListing.pack_start(self.downloadDialog)
         self.window.add(self.pannableListing)
         self.window.show_all()
 
     def buttonFeedClicked(widget, button, self, window, key):
-        disp = DisplayFeed(self.listing, self.listing.getFeed(key), self.listing.getFeedTitle(key), key)
+        disp = DisplayFeed(self.listing, self.listing.getFeed(key), self.listing.getFeedTitle(key), key, self.config)
         disp.connect("feed-closed", self.onFeedClosed)
 
     def onFeedClosed(self, object, key):
-        self.buttons[key].set_text(self.listing.getFeedTitle(key), self.listing.getFeedUpdateTime(key) + " / " 
-                            + str(self.listing.getFeedNumberOfUnreadItems(key)) + " Unread Items")
-        self.buttons[key].show()
+        self.displayListing()
+        #self.buttons[key].set_text(self.listing.getFeedTitle(key), self.listing.getFeedUpdateTime(key) + " / " 
+        #                    + str(self.listing.getFeedNumberOfUnreadItems(key)) + " Unread Items")
+        #self.buttons[key].show()
      
     def run(self):
         self.window.connect("destroy", gtk.main_quit)
         gtk.main()
         self.listing.saveConfig()
 
+    def prefsClosed(self, *widget):
+        self.orientation.set_mode(self.config.getOrientation())
+        self.checkAutoUpdate()
+
+    def checkAutoUpdate(self, *widget):
+        if self.config.isAutoUpdateEnabled():
+            if not self.autoupdate:
+                self.autoupdateId = gobject.timeout_add(int(self.config.getUpdateInterval()*3600000), self.automaticUpdate)
+                self.autoupdate = True
+        else:
+            if self.autoupdate:
+                gobject.source_remove(self.autoupdateId)
+                self.autoupdate = False
+
+    def automaticUpdate(self, *widget):
+        # Need to check for internet connection
+        # If no internet connection, try again in 10 minutes:
+        # gobject.timeout_add(int(5*3600000), self.automaticUpdate)
+        self.button_update_clicked(None, None)
+        return True
 
 if __name__ == "__main__":
     gobject.signal_new("feed-closed", DisplayFeed, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,))
     gobject.signal_new("article-closed", DisplayArticle, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,))
     gobject.signal_new("article-next", DisplayArticle, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,))
     gobject.signal_new("article-previous", DisplayArticle, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,))
+    gobject.signal_new("download-done", DownloadBar, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,))
     gobject.threads_init()
     if not isdir(CONFIGDIR):
         try:
